﻿'''<summary>eine typisierte BindingSource mit interner Liste</summary>
'''<typeparam name="T">der übergebende Datentyp, der bei New hinterlegt, bei Add benötigt und bei Current zurückgegeben wird</typeparam>
'''<remarks>Um dieses Control auf einem Form zu verwenden, muss von dieser einfach geerbt werden, z.B.:
'''<code>
'''Public Class MyTypedBindingSource : Inherits TypedBindingSource(Of MyOwnClass) : End Class
'''</code>
''' </remarks>
Public MustInherit Class TypedBindingSource(Of T) : Inherits BindingSource

	Private ReadOnly InternalList As New System.ComponentModel.BindingList(Of T)

	'''<summary>erstellt eine typisierte BindingSource für den Typ T</summary>
	Protected Sub New()
		DataSource = GetType(T)
	End Sub

	'''<summary>fügt der Auflistung ein neues Element vom Typ T hinzu</summary>
	'''<param name="NewItem">das hinzuzufügende Item</param>
	Public Overloads Sub Add(NewItem As T)
		InternalList.Add(NewItem)
	End Sub

	'''<summary>entfernt das momentan ausgewählte Element der Auflistung; wenn die Auflistung leer ist, passiert nichts</summary>
	Public Overloads Sub RemoveCurrent()
		If HasNoCurrent() Then Return

		MyBase.RemoveCurrent()
	End Sub

	'''<summary>ersetzt das ausgewählte Element durch ein anderes</summary>
	'''<param name="Replacement">das Element, welches an der ausgewählten Position den Platz einnehmen soll</param>
	Public Sub ReplaceCurrentWith(Replacement As T)
		If HasNoCurrent() Then Return

		MyBase.Item(Position) = Replacement
		ResetCurrentItem()
	End Sub

	'''<summary>fügt der Auflistung die Elemente einer Liste hinzu</summary>
	'''<param name="List">die Liste der Elemente, die hinzugefügt werden soll</param>
	Public Sub AddRange(List As IEnumerable(Of T))
		List.ForEach(Sub(x) Add(x))
	End Sub

	'''<summary>ersetzt die Auflistungselemente durch andere</summary>
	'''<param name="List">die Liste der Elemente, die sich danach in der Auflistung befinden</param>
	Public Sub ReplaceListContentBy(List As IEnumerable(Of T))
		Clear()
		AddRange(List)
	End Sub

	Private Function IndexIsValid(Index As Integer) As Boolean
		Return Index.IsBetween(0, Count - 1) AndAlso MyBase.Item(Index) IsNot Nothing
	End Function

	Private Sub TypedBindingSource_DataSourceChanged(sender As Object, e As EventArgs) Handles Me.DataSourceChanged
		If Diagnostics.Debugger.IsAttached AndAlso DataSource Is GetType(T) Then
			DataSource = InternalList
		End If
	End Sub

	'''<summary>ruft das Element an der angegebenen Position ab oder legt dieses fest</summary>
	''' <param name="Index">die Position innerhalb der Liste</param>
	''' <returns>das Element an der angegebenen Position</returns>
	Default Public Shadows Property Item(Index As Integer) As T
		Get
			If Not IndexIsValid(Index) Then Return Nothing
			Return DirectCast(MyBase.Item(Index), T)
		End Get
		Set
			If Not IndexIsValid(Index) Then Return
			MyBase.Item(Index) = Value
			ResetBindings(False)
		End Set
	End Property

	'''<summary>ruft das aktuell ausgewählte Item ab oder legt dieses fest</summary>
	Public Shadows Property Current As T
		Get
			Return DirectCast(MyBase.Current, T)
		End Get
		Set
			If IsBindingSuspended OrElse HasNoCurrent OrElse Value.NotExists Then Return
			For i = 0 To Count - 1
				If Item(i).Equals(Value) Then Position = i : Exit For
			Next
		End Set
	End Property

	'''<summary>gibt wieder, ob die TypedBindingSource befüllt ist</summary>
	Public ReadOnly Property HasCurrent() As Boolean
		Get
			Return Position > -1 AndAlso Current.Exists
		End Get
	End Property

	'''<summary>gibt wieder, ob die TypedBindingSource leer ist</summary>
	Public ReadOnly Property HasNoCurrent() As Boolean
		Get
			Return Not HasCurrent
		End Get
	End Property

	'''<summary>gibt wieder, ob die aktuelle Position 0 ist</summary>
	Public ReadOnly Property IsAtListStart() As Boolean
		Get
			Return Position = 0
		End Get
	End Property

	'''<summary>gibt wieder, ob die aktuelle Position am Ende der Liste ist</summary>
	Public ReadOnly Property IsAtListEnd() As Boolean
		Get
			Return Position = List.Count - 1
		End Get
	End Property

End Class